/**
 * @author Matt Spear
 *
 * Simple class that specifies an Edge, using the name of the from and the
 * name to with a transition.  Ordered according to the transition before the 
 * to before the from.
 * 
 * @see Node
 * @see Graph
 */
public class Edge implements Comparable
{
	String trans;
	String from, to;
	String onHeadWork;
	int mvRdHd;
	String putOnWork;
	int mvWrkHd;
	String out;
	/**
	 * Construct a default Edge without any to or from or transition
	 */
	public Edge()
	{
		trans = "";
		to = null;
		from = null;
		onHeadWork = "";
		putOnWork = "";
		out = "";
		mvRdHd = 0;
		mvWrkHd = 0;
	}
	/**
	 * Construct an Edge with the from and to names specified, no transition specified.
	 * 
	 * @param from The name of the node from
	 * @param to The name of the node to
	 */
	public Edge(String from, String to)
	{
		this.from = from;
		this.to = to;
		trans = "";
		onHeadWork = "";
		putOnWork = "";
		out = "";
		mvRdHd = 0;
		mvWrkHd = 0;
	}
	/**
	 * Constructs an Edge with the from, to, and transition specifid.
	 * 
	 * @param from The name of the Node from
	 * @param to The name of the Node to
	 * @param transition The symbol for the transition
	 */
	public Edge(String from, String to, String transition)
	{
		this.from = from;
		this.to = to;
		trans = transition;
		onHeadWork = "";
		putOnWork = "";
		out = "";
		mvRdHd = 0;
		mvWrkHd = 0;
	}
	/**
	 * Constructs an Edge with all parameters specified.
	 * delta = {(from, input, behindHead, to, moveRdHead, putOnWorkTape, moveWorkHead, output)}
	 * 
	 * @param from The name of the Node from
	 * @param to The name of the Node to
	 * @param transition The symbol for the transition
	 * @param onWorkHead The symbol that must be on the work tape
	 * @param moveReadHead The amount to move the read head
	 * @param putOnWorkHead The symbol to place on the work tape
	 * @param moveWorkHead The amount to move the work head
	 * @param output The symbol to output 
	 */
	public Edge(String from, String transition, String onWorkHead, String to, int moveReadHead, String putOnWorkHead, int moveWorkHead, String output)
	{
		this.from = from;
		this.to = to;
		trans = transition;
		onHeadWork = onWorkHead;
		putOnWork = putOnWorkHead;
		out = output;
		mvRdHd = moveReadHead;
		mvWrkHd = moveWorkHead;
	}
	/**
	 * Gets the name of the node from.
	 * 
	 * @return The name of the node from
	 */
	public String getFrom()
	{
		return from;
	}
	/**
	 * Gets the name of the node to.
	 * 
	 * @return The name of the node to
	 */
	public String getTo()
	{
		return to;
	}
	/**
	 * Gets the symbol for the transition.
	 * 
	 * @return the symbol for the transition
	 */
	public String getTransition()
	{
		return trans;
	}
	/**
	 * Gets the symbol for what needs be on the work tape.
	 * 
	 * @return the symbol for what needs be on the work tape
	 */
	public String getOnWorkHead()
	{
		return onHeadWork;
	}
	/**
	 * Gets the number of places to move the read head.
	 * 
	 * @return the number of places to move the read head
	 */
	public int getMoveReadHead()
	{
		return mvRdHd;
	}
	/**
	 * Gets the symbol for what needs be put on the work tape.
	 * 
	 * @return the symbol for what needs be put on the work tape
	 */
	public String getPutOnWork()
	{
		return putOnWork;
	}
	/**
	 * Gets the number of places to move the work head.
	 * 
	 * @return the number of places to move the work head
	 */
	public int getMoveWorkHead()
	{
		return mvWrkHd;
	}
	/**
	 * Gets the symbol for the output.
	 * 
	 * @return the symbol for the output
	 */
	public String getOutput()
	{
		return out;
	}
	/**
	 * Sets the name of the node from.
	 * 
	 * @param f the name of the node fromn
	 */
	public void setNodeFrom(String f)
	{
		from = f;
	}
	/**
	 * Sets the name of the node to
	 * 
	 * @param t the name of the node to
	 */
	public void setNodeTo(String t)
	{
		to = t;
	}
	/**
	 * Sets the symbol for the transition
	 * 
	 * @param t the symbol for the transition
	 */
	public void setTransition(String t)
	{
		trans = t;
	}
	/**
	 * Sets the symbol for what needs be on the work head
	 * 
	 * @param t the symbol for what needs be on the work head
	 */
	public void setOnWorkHead(String t)
	{
		onHeadWork = t;
	}
	/**
	 * Sets the number of places to move the work head
	 * 
	 * @param i the number of places to move the work head
	 */
	public void setMoveWorkHead(int i)
	{
		mvWrkHd = i;
	}
	/**
	 * Sets the number of places to move the read head
	 * 
	 * @param i the number of places to move the read head
	 */
	public void setMoveReadHead(int i)
	{
		mvRdHd = i;
	}
	/**
	 * Sets the symbol for what to place on the work head
	 * 
	 * @param t the symbol for what to place on the work head
	 */
	public void setPutOnWorkHead(String t)
	{
		putOnWork = t;
	}
	/**
	 * Sets the symbol for the output
	 * 
	 * @param t the symbol for the output
	 */
	public void setOutput(String t)
	{
		out = t;
	}
	/**
	 * Tests if this has the transition specified
	 * 
	 * @param t the transition to test for
	 * @return true if the transitions are equivilant false otherwose
	 */
	public boolean hasTransition(String t)
	{
		return t.equals(trans);
	}
	/**
	 * Compares this with the Object specifed
	 * 
	 * @see Edge.equals(Edge e)
	 * @return false if o is not an edge else return is specified by Edge.equals(Edge e)
	 */
	public boolean equals(Object o)
	{
		if (o instanceof Edge)
			return equals((Edge)o);
		return false;
	}
	/**
	 * Tests if the Edge specified is equivilant to this, i.e. have the same to
	 * from and transition.
	 * 
	 * @param e the edge to compare with
	 * @return true if the to, from and transition are the same.
	 */
	public boolean equals(Edge e)
	{
		return trans.equals(e.trans) && from.equals(e.from) && to.equals(e.to) && mvRdHd == e.mvRdHd && mvWrkHd == e.mvWrkHd && onHeadWork.equals(e.onHeadWork) && out.equals(e.out) && putOnWork.equals(e.putOnWork);
	}
	/**
	 * Tests if the name of the node from and the name of the node to and the
	 * transition are equivilant with the ones in this.
	 * 
	 * @param nf the name of the node from
	 * @param nt the name of the node to
	 * @param tr the symbol for the transition
	 * @return true if equivilant
	 */
	public boolean equals(String nf, String nt, String tr)
	{
		return from.equals(nf) && to.equals(nt) && trans.equals(tr);
	}
	/**
	 * Tests for equality using all possible parameters
	 * 
	 * @param nf The from state
	 * @param nt The to state
	 * @param tr The transition
	 * @param wkHd The symbol currently on the work tape
	 * @param mvRdHd The amount to move the Read head
	 * @param pWkHd The symbol to put on the work tape
	 * @param mvWkHd The amount to move the work head
	 * @param output The symbol to output
	 * @return true iff equivilant
	 */
	public boolean equals(String nf, String nt, String tr, String wkHd, int mvRdHd, String pWkHd, int mvWkHd, String output)
	{
		return from.equals(nf) && to.equals(nt) && trans.equals(tr) && wkHd.equals(onHeadWork) && mvRdHd == mvRdHd && pWkHd.equals(putOnWork) && mvWkHd == mvWkHd && output.equals(out);
	}
	/**
	 * Tests if this is a transition rule following the paramaters
	 * 
	 * @param nf the state from
	 * @param tr the transition
	 * @param wkHd the symbol on the work head
	 * @return true iff it is
	 */
	public boolean hasTransition(String nf, String tr, String wkHd)
	{
		return from.equals(nf) && trans.equals(tr) && wkHd.equals(onHeadWork);
	}
	/**
	 * Compares if this is smaller than the Edge or String
	 * specified.  On a String parameter merely compares lixigraphically
	 * the transitions.  On an Edge parameter compares the transition, name
	 * of the node from and the name of the node to.
	 * 
	 * @return the relative ranking of this edge
	 */
	public int compareTo(Object o)
	{
		if (o instanceof Edge)
		{
			Edge e = (Edge)o;
			if (trans.equals(e.trans)
					&& to.equals(e.to)
					&& from.equals(e.from))
				return 0;
			else if (from.equals(e.from) && to.equals(e.to))
				return trans.compareTo(e.trans);
			else if (from.equals(e.from))
				return to.compareTo(e.to);
			else
				return from.compareTo(from);
		}
		else if (o instanceof String)
			return trans.compareTo(o);
		return -1;
	}
	/**
	 * Converts this to a simple string < FROM , TRANSITION , ONWORKHEAD, TO, MOVEREADHEAD, PUTONWORKHEAD, MOVEWORKHEAD, OUTPUT >
	 * 
	 * @return the String representation of this
	 */
	public String toString()
	{
		return "( "
		+ from.toString()
		+ " , "
		+ trans
		+ " , "
		+ onHeadWork
		+ " , "
		+ to.toString()
		+ " , "
		+ mvRdHd
		+ " , "
		+ putOnWork
		+ " , "
		+ mvWrkHd
		+ " , "
		+ out
		+ " ) ";
	}
	/**
	 * Returns a clone of this.
	 * 
	 * @return a clone of this.
	 */
	public Object clone()
	{
		return new Edge(from, trans, onHeadWork, to, mvRdHd, putOnWork, mvWrkHd, out);
	}
}
