import java.util.*;

/**
 * @author Matt Spear
 *
 * Simple LinkedList representation of a Graph.
 * 
 * @see Node
 * @see Edge
 * @see TreeSetNode
 */
public class Graph
{
	protected TreeSet tm; // Collection of TreeSetNodes
	protected ArrayList alpha, gamma, delta; // The alphabet, only used in output
	
	/**
	 * Constructs an empty Graph
	 *
	 */
	public Graph()
	{
		tm = new TreeSet();
		alpha = null;
	}
	
	/**
	 * Adds the node specified.
	 * @param n the node to add
	 */
	public void insertNode(Node n)
	{
		if(n != null)
		{	
			tm.add(new TreeSetNode(n, new TreeSet()));
		}
	}
	/**
	 * Adds the Edge specified.
	 * @param e the edge to add
	 */
	public void insertEdge(Edge e)
	{
		if(e == null)
			return;
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(e.getFrom()))
			{	
				tn.tm.add(e);
				return;
			}
		}
	}
	/**
	 * Sets the alphabet to use for output of the graph.
	 * @param alphabet the ArrayList of Strings that is the alphabet
	 */
	public void setAlphabet(ArrayList alphabet)
	{
		alpha = alphabet;
	}
	/**
	 * Sets the gamma to use for output of the graph.
	 * @param alphabet the ArrayList of Strings that is the alphabet
	 */
	public void setGamma(ArrayList alphabet)
	{
		gamma = alphabet;
	}
	/**
	 * Sets the delta to use for output of the graph.
	 * @param alphabet the ArrayList of Strings that is the alphabet
	 */
	public void setDelta(ArrayList alphabet)
	{
		delta = alphabet;
	}
	/**
	 * Get the node with the name speciied
	 * @param s the name of the Node to get
	 * @return the node specified, or null if it is non-existant in this
	 */
	public Node getNode(String s)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(s))
				return (Node)tn.n.clone();
		}
		return null;
	}
	
	/**
	 * Gets the root of the graph.
	 * @return the nam of the root of the graph
	 */
	public String getRoot()
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.isRoot())
				return tn.n.toString();
		}
		return null;
	}
	
	/**
	 * Sets the node specified by name to be an accepting state.
	 * @param n the name of the node to set to accepting
	 * @return true if the node exists, false otherwise
	 */
	public boolean setAccepting(String n)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n))
			{	
				tn.n.makeAccepting();
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Tests to see if the node specified by name is an accepting node
	 * @param n the name of the node to test for accepting
	 * @return true iff the node specified is an accepting node
	 */
	public boolean isAccepting(String n)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n))
			{	
				return tn.n.isAccepting();
			}
		}
		return false;
	}
	
	/**
	 * Sets the node specified to be a root node.
	 * @param n the name of the node to become a root node
	 * @return true iff the node exists in this graph
	 */
	public boolean setRoot(String n)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n))
			{	
				tn.n.makeRoot();
				return true;
			}
		}
		return false;
	}

	/**
	 * Tests if the node specified by name exists in this graph.
	 * @param n the name of the node to test for existance
	 * @return true iff the node is in the graph
	 */
	public boolean containsNode(String n)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n))
				return true;
		}
		return false;
	}
	/**
	 * Gets the number of nodes in the graph
	 * @return the number of nodes in the graph
	 */
	public int numNodes() { return tm.size(); }
		
	/**
	 * Gets an Iterator over the Edges of the Node specified by name.
	 * 
	 * @param n the name of the node to retrieve the Edges for
	 * @return An Iterator over the edges of the Node specified
	 */
	public Iterator getEdges(String n)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n))
				return tn.tm.iterator();
		}
		return null;
	}
	
	/**
	 * Gets an Iterator over the Edges of the Node specified.
	 * 
	 * @param n the name of the node to retrieve the Edges for
	 * @return An Iterator over the edges of the Node specified
	 */
	public Iterator getEdges(Node n1)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n1))
				return tn.tm.iterator();
		}
		return null;
	}
	
	/**
	 * Tests to see if the Edge specified exists in this.
	 * @param e the Edge to test for existance
	 * @return true iff the edge exists in this graph.
	 */
	public boolean containsEdge(Edge e)
	{
		if(e == null)
			return false;
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(e.getFrom()))
				return tn.tm.contains(e);
		}
		return false;
	}
	/**
	 * Tests to see if this graph contains an edge with the transition specified.
	 * @param trans the transition to see if any edge in this graph has.
	 * @return true iff an edge in this graph has the transition
	 */
	public boolean containsEdge(String trans)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(trans))
				return tn.tm.contains(trans);
		}
		return false;
	}
	/**
	 * Gets the edge specified
	 * @param nf the node from
	 * @param nt the node to
	 * @param tr the transition
	 * @param wkHd under work head
	 * @param mvRdHd amount to move read head
	 * @param pWkHd put on work head
	 * @param mvWkHd amount to move work head
	 * @param output the output
	 * @return the edge specified or null if no edge matches
	 */
	public Edge getEdge(String nf, String nt, String tr, String wkHd, int mvRdHd, String pWkHd, int mvWkHd, String output)
	{
		Iterator it = getEdges(nf);
		Edge e;
		while(it.hasNext())
		{
			e = (Edge)it.next();
			if(e.equals(nf,nt, tr, wkHd, mvRdHd, pWkHd, mvWkHd, output))
				return e;
		}
		return null;
	}
	
	/**
	 * Getse the name of the next node after using the transition specified 
	 * @param f the node to start from
	 * @param tr the transition to use to traverse to the next node
	 * @param wkHd what is on the Work Head
	 * @return the next node, or null if there is no edge with the specified transition
	 */
	public Edge getNext(String f, String tr, String wkHd)
	{
		if(f == null)
			return null;
		Iterator it = getEdges(getNode(f));
		if(it == null)
			return null;
		Edge e;
		while(it.hasNext())
		{
			e = (Edge)it.next();
			if(e.hasTransition(f, tr, wkHd))
				return e;
		}
		return null;
	}
	
	/** 
	 * Marks the node specified as visited.
	 * @param n the name of the node to mark as visited
	 */
	public void visitNode(String n)
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{	
			tn = (TreeSetNode)it.next();
			if(tn.n.equals(n))
				tn.n.visit();
		}
	}
	
	public void resetNodes()
	{
		Iterator it = tm.iterator();
		TreeSetNode tn;
		while(it.hasNext())
		{
			tn = (TreeSetNode)it.next();
			tn.n.unVisit();
		}
	}
	
	/**
	 * Returns the FSA configuration description of the graph.
	 * M = <Q,\Sigma, \Gamma, \Delta, \delta, START, BLANK, F>
	 */
	public String toString()
	{
		StringBuffer m = new StringBuffer("M = <Q,\\Sigma,\\Gamma,\\Delta,\\delta," + getRoot() + "," + UTM.blank + ",F>\n");
		StringBuffer q = new StringBuffer("Q = {");
		int qq = q.length();
		StringBuffer sig = new StringBuffer("\\Sigma = {");
		int sigq = sig.length();
		StringBuffer gam = new StringBuffer("\\Gamma = {");
		int gamq = gam.length();
		StringBuffer delC = new StringBuffer("\\Delta = {");
		int delCq = delC.length();
		StringBuffer del = new StringBuffer("\\delta = {");
		int delq = del.length();
		StringBuffer f = new StringBuffer("F = {");
		int fq = f.length();
		Iterator it;
		Iterator i = tm.iterator();
		TreeSetNode tn;
		for(; i.hasNext();)
		{
			tn = (TreeSetNode)i.next();
			q.append(tn.n.toString() + ",");
			if(tn.n.isAccepting())
				f.append(tn.n.toString() + ",");
			it = getEdges(tn.n);
			if(it == null)
				continue;
			for(; it.hasNext(); )
			{
				Edge e = (Edge)it.next();
				del.append("(" 
							+ e.getFrom() 
							+ "," 
							+ e.getTransition() 
							+ "," 
							+ e.getOnWorkHead()
							+ ","
							+ e.getTo()
							+ ","
							+ e.getMoveReadHead()
							+ ","
							+ e.getPutOnWork()
							+ ","
							+ e.getMoveWorkHead()
							+ ","
							+ e.getOutput()
							+ "),");
			}
		}
		if(del.length() > delq)
		{	
			del.deleteCharAt(del.length()-1);
		}
		del.append("}\n");
		if(q.length() > qq)
			q.deleteCharAt(q.length()-1); 
		q.append("}\n");
		if(alpha != null)
		{	
			for(i = alpha.iterator(); i.hasNext(); )
			{
				String s = i.next().toString();
				sig.append(s + ",");
			}
			if(sig.length() > sigq)
			{	
				sig.deleteCharAt(sig.length()-1);
			}
			sig.append("}\n");
		}
		else
		{
			sig.append("UNKNOWN}\n");
		}
		if(gamma != null)
		{	
			for(i = gamma.iterator(); i.hasNext(); )
			{
				String s = i.next().toString();
				gam.append(s + ",");
			}
			if(gam.length() > gamq)
			{	
				gam.deleteCharAt(gam.length()-1);
			}
			gam.append("}\n");
		}
		else
		{
			gam.append("UNKNOWN}\n");
		}
		if(delta != null)
		{	
			for(i = delta.iterator(); i.hasNext(); )
			{
				String s = i.next().toString();
				delC.append(s + ",");
			}
			if(delC.length() > delCq)
			{	
				delC.deleteCharAt(delC.length()-1);
			}
			delC.append("}\n");
		}
		else
		{
			delC.append("UNKNOWN}\n");
		}
		if(f.length() > fq)
			f.deleteCharAt(f.length()-1); 
		f.append("}\n");
		return m.toString() + q.toString() + sig.toString() + gam.toString() + delC.toString() + del.toString() + f.toString();
	}
}
