import java.util.*;
import java.io.*;

/**
 * @author Matt Spear
 * 
 * The class that handles all UTM manipulations.
 * 
 * @see Graph
 * @see FSASimApplet
 */
public class UTM
{
	protected Graph g; // The graph of the UTM
	protected String input; // Input
	protected int inputAt; // character at in the input
	protected StringBuffer work; // The work head
	protected int workAt; // Where at in work head
	protected String on; // the name of the node on now
	protected String output; // The output
	protected boolean done, out; // if done or out of input
	protected String read; // what has been read so far
	protected ArrayList alpha, gamma, delta; // the alphabet
	public static String blank; // The blank symbol (cannot be modified as String is immutable)
	
	/**
	 * Constructs a empty UTM.  Must add with the mutators.
	 */
	public UTM()
	{
		g = new Graph();
		//loc = -1;
		inputAt = -1;
		on = null;
		done = false;
		out = false;
		read = "";
		workAt = 0;
		work = new StringBuffer();
		output = "";
		blank = "";
		alpha = new ArrayList();
		gamma = new ArrayList();
		delta = new ArrayList();
	}
	
	/**
	 * Constructs an empty UTM except the input is now specified.
	 * 
	 * @param input the input to the machine
	 */
	public UTM(String input)
	{
		this();
		if(input != null)
		{
			this.input = input;
			inputAt = 0;
		}
	}
	
	/**
	 * Gets the output of this machine
	 * @return the output of the machine
	 */
	public String getOutput() { return output; }
	
	/**
	 * Tests if the UTM is done running, either rejected or accepted the input.
	 * 
	 * @return true if done with the input false otherwise.
	 */
	public boolean isDone()
	{
		return done;
	}
	
	/**
	 * Tests if the UTM has no nodes/states,
	 * 
	 * @return true iff no nodes/states have been added.
	 */
	public boolean isEmpty()
	{
		return g.numNodes() == 0;
	}
	/**
	 * Tests to see if the input has been specified.
	 * 
	 * @return true if the input has been specified.
	 */
	public boolean hasInput()
	{
		return !(input == null);
	}
	/**
	 * Sets the input to use in the simulation.
	 * @param s the input to read through
	 * @throws UTMException if the String contains illegal characters, not in the alphabet.
	 */
	public void setInput(String s) throws UTMException
	{
		if (s == null)
			return;
		for (int i = 0; i < s.length(); i++)
		{
			if (!isInAlphabet("" + s.charAt(i)))
				throw new UTMException(
					"Invalid symbol in input:\t" + s.charAt(i));
		}
		this.input = s;
		//loc = 0;
		inputAt = 0;
	}
	public void setBlank(String b)
	{
		blank = b;
	}
	/**
	 * Sets the alphabet (Sigma) for the FSA.
	 * 
	 * @param alphabet the alphabet for the FSA, must be an ArrayList of Strings
	 */
	public void setAlphabet(ArrayList alphabet)
	{
		if (alphabet != null)
		{
			alpha = alphabet;
			g.setAlphabet(alphabet);
		}
	}
	/**
	 * Sets the alphabet (Gamma) for the FSA.
	 * 
	 * @param alphabet the alphabet for the FSA, must be an ArrayList of Strings
	 */
	public void setGamma(ArrayList alphabet)
	{
		if (alphabet != null)
		{
			gamma = alphabet;
			g.setGamma(alphabet);
		}
	}
	/**
	 * Sets the alphabet (Delta) for the UTM.
	 * 
	 * @param alphabet the alphabet for the UTM, must be an ArrayList of Strings
	 */
	public void setDelta(ArrayList alphabet)
	{
		if (alphabet != null)
		{
			delta = alphabet;
			g.setDelta(alphabet);
		}
	}
	/**
	 * Gets the alphabet allowed in this
	 * @return an ArrayList of Strings that has the symbols in this UTM Sigma 
	 */
	public ArrayList getAlphabet()
	{
		return alpha;
	}
	public void addAlphabet(String s)
	{
		alpha.add(s);
	}
	/**
	 * Tests if the specified symbol is in the alphabet
	 * 
	 * @param c the sybmol to test for
	 * @return true iff the symbol is in the alphabet
	 */
	public boolean isInAlphabet(String c)
	{
		if(c.equals("$") || c.equals("\\cent"))
				return true;
		return alpha.contains(c);
	}

	/**
	 * Preforms a single step if possible, i.e. moves from one state to the next.
	 * 
	 * @return true if the step caused an accepting computation
	 * @throws RejectionException if there is no step (rejecting computation)
	 * @throws UTMException if there is no input
	 */
	public boolean step() throws RejectionException, UTMException
	{
		if(done)
			return true;
		if(on == null)
			return false;
		if(input == null)
			throw new UTMException("Input must be spcified");
		String at;
		if(inputAt >= input.length())
		{	
			at = "$";
		}
		else if(inputAt < 0)
		{	
			at = "\\cent";
		}
		else
		{	
			at = "" + input.charAt(inputAt);
		}
		
		Edge t = null;
		if(workAt >= 0 && workAt < work.length())
		{	
			t = g.getNext(on, "" + at, "" + work.charAt(workAt));
		}
		else
		{	
			t = g.getNext(on, "" + at, blank);
		}
		if(t == null)
		{
			if(g.getNode(on).isAccepting() && inputAt >= input.length())
			{
				done = true;
				return true;
			}
			throw new RejectionException("UTM failed at input marker:\t" + at);
		}
		on = t.getTo();
		inputAt += t.getMoveReadHead();
		if(!t.getOutput().equals("\\epsilon"))
			output += t.getOutput();
		if(inputAt >= input.length())
		{	
			read = input + on;
		}
		else if(inputAt < 0)
		{	
			read = on + input;
		}
		else
		{	
			read = input.substring(0, inputAt) + on + input.substring(inputAt);
		}
		if(workAt < work.length() && workAt >= 0)
		{
			if(t.getPutOnWork().equals(blank))
				work.deleteCharAt(workAt);
			else
				work.insert(workAt, t.getPutOnWork());
		}
		else if(workAt < 0)
			work = new StringBuffer(blank + work.toString());
		else
		{
			if(!t.getPutOnWork().equals(blank))
				work.append(t.getPutOnWork());
		}
		workAt += t.getMoveWorkHead();
		return false;
	}
	/**
	 * Adds a node/state to the UTM.  If the Node specified is a root and
	 * a root already exists throws a RootDeclaredException, if the Node
	 * exists in this also throws a RootDeclaredException
	 * @param n the node to add
	 * @throws RootDeclaredException if the node exists in this already, or there is already a root and the node specified is declared to be a root.
	 */
	public void addNode(String n) throws RootDeclaredException
	{
		if (g.containsNode(n))
			throw new RootDeclaredException("Cannot have same node twice");
		Node no = new Node(n);
		g.insertNode(no);
		if (no.isRoot())
		{
			if (on != null)
				throw new RootDeclaredException("Root already exists");
			on = n;
		}
	}
	/**
	 * Sets the root for this.
	 * @param s the name of the node to be root.
	 * @throws RootDeclaredException if the Root already is specified.
	 * @throws InvalidRootException 
	 */
	public void setRoot(String s)
		throws RootDeclaredException, InvalidRootException
	{
		if (on != null)
			throw new RootDeclaredException("Root already exists");
		g.setRoot(s);
		on = s;
	}
	/**
	 * Sets the name of the node specified to be an accepting state/node.
	 *  
	 * @param s the name of the node to set to accepting.
	 */
	public void setAccepting(String s)
	{
		g.setAccepting(s);
	}
	/**
	 * Adds the transition from the name of the node specified to the name of the node specified
	 * with the transition symbol specified.
	 * @param f the name of the node from
	 * @param t the name of the node to
	 * @param trans the symbol for the transition
	 * @throws UTMException  if the transition symbol is not in the alphabet
	 * @see DFS
	 */
	public void addTransition(String f, String trans, String onWorkHead, String t, int moveReadHead, String putOnWorkHead, int moveWorkHead, String output) throws UTMException
	{
		if (!isInAlphabet(trans))
			throw new UTMException("Transition is not in alphabet:\t" + trans);
		g.insertEdge(new Edge(f, trans, onWorkHead, t, moveReadHead, putOnWorkHead, moveWorkHead, output));
	}
	public void addTransition(Edge e) throws UTMException
	{
		if (!isInAlphabet(e.getTransition()))
			throw new UTMException("Transition is not in alphabet:\t" + e.getTransition());
		g.insertEdge(e);
	}
	/**
	 * Resets the UTM, sets on back to root, makes sure is not done and clears the input.
	 *
	 */
	public void reset()
	{
		on = g.getRoot();
		done = false;
		out = false;
		input = null;
		read = "";
		inputAt = 0;
		work.setLength(0);
		workAt = 0;
		output = "";
	}
	/**
	 * Returns the specification of the machine.
	 * M = <Q,\Sigma,\delta,START,F>
	 */
	public String toString()
	{
		return g.toString();
	}
	/**
	 * Dumps the instantanious description of the UTM out in the return value
	 * using the uqv syntax, u is the input read, q is the state name and v is
	 * the input left to read.
	 * @return the String of the instantanious configuration
	 */
	public String dump()
	{
		StringBuffer ret = new StringBuffer("< " + (read.length() == 0 ? on + input : read) + " , ");
		if(workAt >= 0 && workAt < work.length())
			ret.append(work.substring(0,workAt) + on + work.substring(workAt));
		else if(workAt < 0)
			ret.append(on + work);
		else
			ret.append(work + on);
		ret.append(" , " + output + " >");
		return ret.toString();
	}
	/**
	 * Quick tester.
	 * @param args not used
	 */
	public static void main(String[] args)
	{	
		try
		{
			parser p = new parser(new Scanner(new FileReader("machine.txt")));
			
			UTM u =(UTM) p.parse().value;
			
			u.setInput("a");
			
			System.out.println("\n\n\n" + u.toString());
			
			System.out.println(u.dump());
			boolean b = false;
			while(!u.isDone())
			{
				b = u.step();
				System.out.println(u.dump());
			}
			if(b == true)
				System.out.println("Accepted");
		}
		catch(Exception e) { e.printStackTrace(); }
	}
}
