/*
  This example comes from a short article series in the Linux 
  Gazette by Richard A. Sevenich and Christopher Lopes, titled
  "Compiler Construction Tools". The article series starts at

  http://www.linuxgazette.com/issue39/sevenich.html

  Small changes and updates to newest JFlex+Cup versions 
  by Gerwin Klein
*/

/*
  Commented By: Christopher Lopes
  File Name: lcalc.flex
  To Create: > jflex lcalc.flex

  and then after the parser is created
  > javac Lexer.java
*/
   
/* --------------------------Usercode Section------------------------ */
   
import java_cup.runtime.*;
      
%%
   
/* -----------------Options and Declarations Section----------------- */
   
/* 
   The name of the class JFlex will create will be Scanner.
   Will write the code to the file Scanner.java. 
*/
%class Scanner

/*
  The current line number can be accessed with the variable yyline
  and the current column number with the variable yycolumn.
*/
%line
%column
    
/* 
   Will switch to a CUP compatibility mode to interface with a CUP
   generated parser.
*/
%cup
   
/*
  Declarations
   
  Code between %{ and %}, both of which must be at the beginning of a
  line, will be copied verbatim  into the Scanner class source.
  Here you declare member variables and functions that are used inside
  scanner actions.  
*/
%{   
    /* To create a new java_cup.runtime.Symbol with information about
       the current token, the token will have no value in this
       case. */
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }
    
    /* Also creates a new java_cup.runtime.Symbol with information
       about the current token, but this object has a value. */
    private Symbol symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }
%}
   

/*
  Macro Declarations
  
  These declarations are regular expressions that will be used latter
  in the Lexical Rules Section.  
*/
   
/* A line terminator is a \r (carriage return), \n (line feed), or
   \r\n. */
LineTerminator = \r|\n|\r\n
   
/* White space is a line terminator, space, tab, or line feed. */
WhiteSpace     = {LineTerminator} | [ \t\f]
   

/* a state is just q_y  where y is a natural number */
dec_state = q_[0-9]+

/* a symbol is just an alphanumeric symbol */
dec_symbol = [A-Za-z0-9]
/* a direction is either -1, 0 or +1 */
dir_symbols = \-1|0|\+1
/* [\-\+]?[0-1] */

   
%%
/* ------------------------Lexical Rules Section---------------------- */
   
/*
   This section contains regular expressions and actions, i.e. Java
   code, that will be executed when the scanner matches the associated
   regular expression. */
   
   /* YYINITIAL is the state at which the lexer begins scanning.  So
   these regular expressions will only be matched if the scanner is in
   the start state YYINITIAL. */
   
<YYINITIAL> {
   
   /* return the token LPAREN ,RPAREN or COMMA  declared in the class sym */
	"("		{ return symbol(sym.LPAREN);}
	"<"		{ return symbol(sym.LBRACKET);}
	"{"		{ return symbol(sym.LBRACE);}
	")"		{ return symbol(sym.RPAREN);}
	">"		{ return symbol(sym.RBRACKET);}
	"}"		{ return symbol(sym.RBRACE);}
	","		{ return symbol(sym.COMMA);}
	"="		{ return symbol(sym.EQUALS);}
	"\\Sigma"	{ return symbol(sym.SIGMA);}
	"\\delta"	{ return symbol(sym.DELTA);}
	"\\Delta"	{ return symbol(sym.UDELTA);}
	"\\Gamma"	{ return symbol(sym.GAMMA);}
   	"\\epsilon"	{ return symbol(sym.SYMB, new String(yytext()));}
	"\\cent" 	{ return symbol(sym.SYMB, new String(yytext()));}
	"$"		{ return symbol(sym.SYMB, new String(yytext()));}
    /* If a state  is found print it out, return the token STATE that represents 
	the state and return the string of the state. */
    {dec_state}      { return symbol(sym.STATE, new String(yytext())); }
   
    /* If a symbol is found, print it out, return the token SYMB
       that represents a symbol and the character of the symbol */
    {dec_symbol}       { return symbol(sym.SYMB, new String(yytext()));}
    {dir_symbols} 	{ return symbol(sym.DIR, new String(yytext()));}   
    /* Don't do anything if whitespace is found */
    {WhiteSpace}       { /* just skip what was found, do nothing */ }   
}
    

/* No token was found for the input so through an error.  Print out an
   Illegal character message with the illegal character that was found. */
[^]                    { throw new RuntimeException("Illegal character <"+yytext()+">"); }












